#IF 0  ' 

    CPUID test program for INTEL processors.

#ENDIF ' 

    %usemacros = 1

    #include "\pbwin90\include\win32api.inc"

    TYPE X86ST
      sse42 as DWORD
      sse41 as DWORD
      sse3  as DWORD
      sse2  as DWORD
      sse   as DWORD
      mmx   as DWORD
    END TYPE

' 

FUNCTION PBmain as LONG

    LOCAL x86 as X86ST

    If iscpuid = 1 Then
      StdOut "CPUID is supported"
    Else
      StdOut "Processor does not support CPUID"
      Exit FUNCTION
    End If

    StdOut Get_Brand_String                 ' display the brand string

    StdOut preparse$(get_cpu_id_string)     ' clean up formatting on return string.

    get_x86_instruction_sets ByVal varPtr(x86)


    If x86.sse42 = 1 Then
      StdOut "SSE4.2 is supported"
    Else
      StdOut "SSE4.2 is not supported"
    End if

    If x86.sse41 = 1 Then
      StdOut "SSE4.1 is supported"
    Else
      StdOut "SSE4.1 is not supported"
    End if

    If x86.sse3 = 1 Then
      StdOut "SSE3 is supported"
    Else
      StdOut "SSE3 is not supported"
    End if

    If x86.sse2 = 1 Then
      StdOut "SSE2 is supported"
    Else
      StdOut "SSE2 is not supported"
    End if

    If x86.sse = 1 Then
      StdOut "SSE is supported"
    Else
      StdOut "SSE is not supported"
    End if

    If x86.mmx = 1 Then
      StdOut "MMX is supported"
    Else
      StdOut "MMX is not supported"
    End if

    Stdout "Press any key to exit ...."
    Do
      Sleep 1
    Loop while inkey$ = ""

    FUNCTION = 0

End FUNCTION

' 

FUNCTION iscpuid as LONG

  ' -------------------------------
  ' bit 21 of EFLAGS test for CPUID
  ' -------------------------------
    #REGISTER NONE

    ! pushfd
    ! pop eax

    ! mov ecx, eax
    ! xor eax, &B00000000001000000000000000000000       ; set bit 21
    ! push eax
    ! popfd
    ! pushfd
    ! pop eax

    ! xor eax, ecx              ; test if its changed
    ! jz NotChanged
    FUNCTION = 1
    Exit FUNCTION

  NotChanged:
    FUNCTION = 0

END FUNCTION

'  

FUNCTION Get_Brand_String as STRING

    #REGISTER NONE

    LOCAL pBuffer as DWORD
    buffer$ = space$(12)
    pBuffer = StrPtr(buffer$)

    ! xor eax, eax
    ! cpuid

    ! mov eax, pBuffer                    ; load buffer address into eax

    ! mov [eax],    ebx                   ; write results to buffer
    ! mov [eax+4],  edx
    ! mov [eax+8],  ecx

    FUNCTION = buffer$

END FUNCTION

'  

FUNCTION get_cpu_id_string as STRING

    #REGISTER NONE

    LOCAL basicinf as DWORD
    LOCAL extfunc as DWORD
    LOCAL pBuffer as DWORD
    buffer$ = space$(48)
    pBuffer = StrPtr(buffer$)

    ! xor eax, eax                              ; call basic info function
    ! cpuid
    ! mov basicinf, eax

    ! mov eax, &H80000000                         ; call extfunc info function
    ! cpuid
    ! mov extfunc, eax

    ! cmp eax, &H80000004
    ! je getID

    ! cmp eax, &H80000008
    ! je getID

    If basicinf = &H00000001 Then
      FUNCTION = "i486 or early Pentium Processor"
      EXIT FUNCTION
    ElseIf basicinf = &H00000002 Then
      FUNCTION = "Pentium Pro, II or Celeron Processor"
      EXIT FUNCTION
    ElseIf basicinf = &H00000003 Then
      FUNCTION = "Pentium III Processor"
      EXIT FUNCTION
    Else
      FUNCTION = "Cannot Identify x86 Processor"
      EXIT FUNCTION
    End if

  getID:
    ! mov esi, pBuffer                    ; load buffer address into ESI
   
    ! mov eax, &H80000002                 ; extfunc function 2
    ! cpuid

    ! mov [esi],    eax                   ; write results to buffer
    ! mov [esi+4],  ebx
    ! mov [esi+8],  ecx
    ! mov [esi+12], edx

    ! mov eax, &H80000003                 ; extfunc function 3
    ! cpuid

    ! mov [esi+16], eax                   ; write results to buffer
    ! mov [esi+20], ebx
    ! mov [esi+24], ecx
    ! mov [esi+28], edx

    ! mov eax, &H80000004                 ; extended function 4
    ! cpuid

    ! mov [esi+32], eax                   ; write results to buffer
    ! mov [esi+36], ebx
    ! mov [esi+40], ecx
    ! mov [esi+44], edx

    FUNCTION = buffer$

END FUNCTION

' 

SUB get_x86_instruction_sets(ByVal pStruct as DWORD)

    #REGISTER NONE

  ' mov eax, 80000001h

  ' bit 20 ecx = sse4.2     intel
  ' bit 19 ecx = sse4.1     intel
  ' bit  0 ecx = sse3       intel   + amd
  ' bit 26 edx = sse2       intel   + amd
  ' bit 25 edx = sse        intel   + amd
  ' bit 23 edx = mmx        intel   + amd

    LOCAL ecx_  as DWORD
    LOCAL edx_  as DWORD
    LOCAL rval  as DWORD

    ! mov esi, pStruct

    ! mov eax, 1
    ! cpuid

    ! mov ecx_, ecx                                   ; store ECX results in variable
    ! mov edx_, edx                                   ; store EDX results in variable

    ! mov eax, ecx_
    ! and eax, &B00000000000100000000000000000000     ; test bit 20 (INTEL sse4.2)
    ! ror eax, 20

    ! cmp eax, 1
    ! jne nxt1
    ! mov DWORD PTR [esi], 1
    ! jmp out1
  nxt1:
    ! mov DWORD PTR [esi], 0
  out1:

    ! mov eax, ecx_
    ! and eax, &B00000000000010000000000000000000     ; test bit 19 (INTEL sse4.1)
    ! ror eax, 19

    ! cmp eax, 1
    ! jne nxt2
    ! mov DWORD PTR [esi+4], 1
    ! jmp out2
  nxt2:
    ! mov DWORD PTR [esi+4], 0
  out2:

    ! mov eax, ecx_
    ! and eax, &B00000000000000000000000000000001     ; test bit 0 (INTEL sse3)

    ! cmp eax, 1
    ! jne nxt3
    ! mov DWORD PTR [esi+8], 1
    ! jmp out3
  nxt3:
    ! mov DWORD PTR [esi+8], 0
  out3:

    ! mov eax, edx_
    ! and eax, &B00000100000000000000000000000000     ; test bit 26 (INTEL sse2)
    ! ror eax, 26

    ! cmp eax, 1
    ! jne nxt4
    ! mov DWORD PTR [esi+12], 1
    ! jmp out4
  nxt4:
    ! mov DWORD PTR [esi+12], 0
  out4:

    ! mov eax, edx_
    ! and eax, &B00000010000000000000000000000000     ; test bit 25 (INTEL sse)
    ! ror eax, 25

    ! cmp eax, 1
    ! jne nxt5
    ! mov DWORD PTR [esi+16], 1
    ! jmp out5
  nxt5:
    ! mov DWORD PTR [esi+16], 0
  out5:

    ! mov eax, edx_
    ! and eax, &B00000000100000000000000000000000     ; test bit 23 (INTEL mmx)
    ! ror eax, 23

    ! cmp eax, 1
    ! jne nxt6
    ! mov DWORD PTR [esi+20], 1
    ! jmp out6
  nxt6:
    ! mov DWORD PTR [esi+20], 0
  out6:

END SUB

' 

FUNCTION preparse$(src$)

  ' --------------------------------------------------
  ' reformat a string for use with the PARSE$ function
  ' --------------------------------------------------
    #REGISTER NONE

    LOCAL pst as DWORD
    LOCAL sln as DWORD

    pst = StrPtr(src$)

    ! mov esi, pst
    ! xor ebx, ebx                      ' prevent stall in BL below
    ! sub esi, 1
    ! mov edi, pst
  trm:                                  ' trim leading tabs and spaces
    ! add esi, 1
    ! cmp BYTE PTR [esi], 32
    ! je trm
    ! cmp BYTE PTR [esi], 9
    ! je trm
    ! sub esi, 1

  ' =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

  stlp:
    ! add esi, 1
    ! mov bl, [esi]
    ! cmp bl, 9
    ! jne nxt1
    ! mov bl, 32                        ' replace tabs with spaces
  nxt1:
    ! cmp bl, 32
    ! jne nxt2
    ! cmp BYTE PTR [esi+1], 32          ' test for next space
    ! je overit
    ! cmp BYTE PTR [esi+1], 9           ' test for next tab
    ! je overit
  nxt2:
    ! mov [edi], bl
    ! add edi, 1
  overit:
    ! test bl, bl                       ' test for zero AFTER its written.
    ! jnz stlp

  ' =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

    ! sub edi, 1
    ! cmp BYTE PTR [edi-1], 32          ' if last character is a space
    ! jne nxt3
    ! sub edi, 1
  nxt3:

    ! sub edi, pst                      ' length in EDI
    ! mov sln, edi

    FUNCTION = left$(src$,sln)

END FUNCTION

' 



















